"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContextBatch = void 0;
const uuid_1 = require("@lukeed/uuid");
const MAX_EVENT_SIZE_IN_BYTES = 32 * 1024; // 32 KB
const MAX_BATCH_SIZE_IN_BYTES = 480 * 1024; // 480 KB (500 KB is the limit, leaving some padding)
class ContextBatch {
    constructor(maxEventCount) {
        this.id = (0, uuid_1.v4)();
        this.items = [];
        this.sizeInBytes = 0;
        this.maxEventCount = Math.max(1, maxEventCount);
    }
    tryAdd(item) {
        if (this.length === this.maxEventCount)
            return false;
        const eventSize = this.calculateSize(item.context);
        if (eventSize > MAX_EVENT_SIZE_IN_BYTES) {
            // Event exceeds Segment's limits
            return false;
        }
        if (this.sizeInBytes + eventSize <= MAX_BATCH_SIZE_IN_BYTES) {
            this.items.push(item);
            this.sizeInBytes += eventSize;
            return true;
        }
        return false;
    }
    get length() {
        return this.items.length;
    }
    calculateSize(ctx) {
        return encodeURI(JSON.stringify(ctx.event)).split(/%..|i/).length;
    }
    getEvents() {
        const events = this.items.map(({ context }) => context.event);
        return events;
    }
    getContexts() {
        return this.items.map((item) => item.context);
    }
    resolveEvents() {
        this.items.forEach(({ resolver, context }) => resolver(context));
    }
}
exports.ContextBatch = ContextBatch;
//# sourceMappingURL=context-batch.js.map